# クラウドデモWEBアプリ用 RPi-GP40 Pythonプログラム

クラウドデモWEBアプリ用 RPi-GP40 Pythonプログラムの使用方法について説明します。  
Raspberry Piは'Raspberry Pi3 ModelB'、OSは'Raspbian Stretch with desktop(NOOBS:2018-11-13)'で説明します。
プログラムファイルは`GP40_demo.py`です。  

[クラウドWEBアプリケーション](https://github.com/ratocsystems/raspberry-pi/tree/master/demo_web_app)が動作するサーバーへRPi-GP40から取得したデータをPOSTします。  
  
***
## 準備
### Raspberry PiにRPi-GP40を接続
[README.md](https://github.com/ratocsystems/rpi-GP40/blob/master/README.md)を参考に下記の準備をおこなってください。  
- OS(`RASPBIAN`)のインストール
- GPIO40pinのSPI設定
- 'Raspberry Pi'に'RPi-GP40'を接続  
  

### プログラムを実行するディレクトリを作成
1. 'mkdir'コマンドを使って'demo_web_app'という名前のディレクトリを作成します。(ディレクトリ名や作成場所は自由です)
    ```
    $ mkdir demo_web_app  
    ```

1. 'ls'コマンドを実行して'demo_web_app'ディレクトリが作成されていること確認します。
    ```
    $ ls  
    ```

1. 'cd'コマンドで'demo_web_app'ディレクトリに移動します。
    ```
    $ cd demo_web_app  
    ```  
    
### PythonプログラムファイルをGitHubからダウンロード  
GitHubからPythonファイルをダウンロードします。
1. GP40_demo.pyをダウンロード
    ```
    $ wget https://github.com/ratocsystems/raspberry-pi/raw/master/demo_web_python/GP40_demo/GP40_demo.py  
    ```  

1. `ls`コマンドを実行してダウンロードされていることを確認します。
    ```
    $ ls  
    GP40_demo.py
    ```
  
***
## Pythonプログラムファイルについて
  
`GP40_demo.py`  

HTTPのPOST通信によりJSON形式にてアナログ入力データを送信するPythonデモプログラムです。  
下記の処理を行っています。

1. **サーバーURL**  
    デモ運用のAmazon AWS EC2サーバーやローカルサーバーアドレスを記述します。  
    サーバー上では[クラウドWEBアプリケーション](https://github.com/ratocsystems/raspberry-pi/tree/master/demo_web_app)の動作が必要です。 
    EC2サーバーの場合起動のたびにアドレスが変更されるため、記述を確認してくださいです。  
    デモ運用AWSサーバーのURLについては「[計測・制御製品サポートセンター](http://www.ratocsystems.com/mail/index.html)」へ問い合わせ願います。

1. **RPi-GP40の初期設定 init_GP40()**  
    GPIOの初期設定を行います。  
    ※<u>ハードウェアに依存する設定ですので変更しないでください。</u>  
    - GPIOをGPIO番号で指定するように設定
    - 絶縁回路用電源をONに設定  
        電源ON後、安定するまで待ちます。
    - デジタル出力端子を出力に設定し、初期状態をOFF(オープン)にします。
    - デジタル/アラーム入力端子を入力に設定します。

1. **入力レンジ設定 set_adrange(ch, range)**  
    ch0～ch7の入力レンジを初期値（引数なしの場合は`±10V`）に設定します。  

1. **指定chのAD変換データ取得 get_addata(ch)**  
    AD変換結果を12bitデータで返します。  

1. **指定間隔と回数でch0-7のAD変換実行と結果データをpostする post_adc(intv, cnt)**  
    指定間隔intv[sec]と指定回数cnt[回]でアナログ入力値を取得し、POST通信でEC2サーバーへJSON形式にて保存します。
    - 現在時刻取得  
    - 8ch分のアナログ入力値取得  
    - 辞書形式変換  
    - JSON形式でPOST要求実行  
    - レスポンスコード表示  

1. **有線LAN(eth0)のMACアドレス取得 getMAC(interface='eth0')**  
    MACアドレスを取得します。  
    MACアドレスはRaspberry Piデバイスの機器判別用IDとして使用されます。  

1. **引数による直接実行形式**  
    引数の指定があれば、指定された間隔で指定回数アナログ入力を実行し、結果をPOSTします。  

1. **引数なしの場合はメニュー実行形式**  
    引数の指定がなければ、メニュー実行形式でアナログ入力を実行します。  

1. **メニュー表示**  
    次のメニューを表示します。  
    `0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >`

1. **0-7:chレンジ設定 set_adrange(ch, range)**  
    メニューで0～7が入力された場合は、指定chの入力レンジを設定します。  
    `0:±10V 1:±5V 2:±2.5V 3:±1.25V 4:±0.5V 5:0-10V 6:0-5V 7:0-2.5V 8:0-1.25V a:無効`  
    から入力レンジを選択します。`[a:無効]`が選択されたchはAD変換を実行しません。

1. **a:単一アナログ入力 post_adc(0, 1)**  
    間隔0秒で1回アナログ入力を実行し、結果をPOSTします。     

1. **b:連続AD変換 post_adc(interval, cnt)**  
    指定した間隔と回数でアナログ入力を実行し、結果をPOSTします。
    `[CTRL]+[C]` で中断します。

1. **e:終了**  
    `[e:終了]`が選択されるとプログラムを終了します。  

***
## Pythonプログラムの使い方  
`python3`をつけて実行します。  

引数を付けて実行すると、直接形式でアナログ入力値をPOSTします。  
`例) ヘルプ表示後に、ch0を±5V, ch1を0-10V, その他を±10Vに設定し、1秒間隔で5回アナログ入力のPOSTを実行する場合`  
~~~
$ python3 GP40_demo.py -h
usage: メニュー形式でRPi-GP40を制御します

引数を指定することで直接実行が可能です

optional arguments:
  -h, --help            show this help message and exit
  -r [R] [R] [R] [R] [R] [R] [R] [R], --range [R] [R] [R] [R] [R] [R] [R] [R]
                        [R]= 0:±10V 1:±5V 2:±2.5V 3:±1.25V 4:±0.5V 5:0-10V
                        6:0-5V 7:0-2.5V 8:0-1.25V 以外:無効 チャンネル0-7の入力レンジ(0-9)を指定
                        例: -r 0 0 5 5 6 6 6 8
  -t [T], --time [T]    [T]= AD変換間隔(1-1000)[秒]を指定 例: -t 1
  -c [C], --cnt [C]     [C]= AD変換回数(1-1000)[回]を指定 例: -c 100

--------------------------------------------------------------------------
$ python3 GP40_demo.py -r 1 5 0 0 0 0 0 0 -t 1 -c 5
RPi-GP40 WEBアプリデモプログラム
     ±5V ch0:  1.9700[B14]
    0-10V ch1:  2.5575[3FF]
    ±10V ch2:  2.2250[9BD]
    ±10V ch3:  2.2250[9BD]
    ±10V ch4:  2.2250[9BD]
    ±10V ch5:  2.2250[9BD]
    ±10V ch6:  2.2250[9BD]
    ±10V ch7:  2.2250[9BD]
    <Response [201]>
    5/5

$
~~~

引数をつけずに実行すると、メニュー形式でアナログ入力値をPOSTします。  
~~~
$ python3 GP40_demo.py
ch:レンジ= 0:±10V, 1:±10V, 2:±10V, 3:±10V, 4:±10V, 5:±10V, 6:±10V, 7:±10V
0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >
~~~  

- **chレンジを設定する**  
    ch0～7の入力レンジを設定するには、設定するch番号を入力し、入力レンジを指定します。  
    `a:無効` を選択した場合はそのchはAD変換を行ないません。  
    `例) ch0を±5V, ch1を0-10Vに設定する場合`  
    ~~~
    0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >0
    入力レンジ 0:±10V 1:±5V 2:±2.5V 3:±1.25V 4:±0.5V 5:0-10V 6:0-5V 7:0-2.5V 8:0-1.25V a:無効
    ch0 入力レンジ >1
    ch:レンジ= 0:±5V, 1:±10V, 2:±10V, 3:±10V, 4:±10V, 5:±10V, 6:±10V, 7:±10V
    0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >1
    入力レンジ 0:±10V 1:±5V 2:±2.5V 3:±1.25V 4:±0.5V 5:0-10V 6:0-5V 7:0-2.5V 8:0-1.25V a:無効
    ch0 入力レンジ >5
    ch:レンジ= 0:±5V, 1:0-10V, 2:±10V, 3:±10V, 4:±10V, 5:±10V, 6:±10V, 7:±10V
    0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >
    ~~~  

- **単一AD変換を行なう**  
    単一AD変換を行なう場合は `a:単一AD変換` を選択します。  
    ch0からch7を設定されたレンジで計測して、算出した電圧(電流)結果を表示します。  
    ~~~
    0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >a  
     ±5V ch0:  1.9700[B14]
    ±10V ch1:  2.2250[9BD]
    ±10V ch2:  2.2250[9BD]
    ±10V ch3:  2.2250[9BD]
    ±10V ch4:  2.2250[9BD]
    ±10V ch5:  2.2250[9BD]
    ±10V ch6:  2.2250[9BD]
    ±10V ch7:  2.2250[9BD]
    <Response [201]>
    ~~~  

- **連続AD変換を行なう**  
    連続AD変換を行なう場合は `b:連続AD変換` を選択します。  
    ch0からch7を設定されたレンジで計測して、算出した電圧(電流)結果を、指定された間隔[秒]で指定回数[回]連続して表示します。  
    ~~~
    0-7:chレンジ設定, a:単一AD変換, b:連続AD変換, e:終了 >b
     連続AD変換 間隔 1-1000[秒] >1
     連続AD変換 回数 1-1000[回] >10
     ±5V ch0:  2.7850[C5A]
    ±10V ch1: 10.2350[FFF]
    ±10V ch2:  2.0900[9A2]
    ±10V ch3:  1.7450[95D]
    ±10V ch4:  1.4000[918]
    ±10V ch5:  1.0500[8D2]
    ±10V ch6:  0.7000[88C]
    ±10V ch7:  0.3500[846]
    <Response [201]>
    10/10
    ~~~  

- **HTTPレスポンスステータスコードについて**  
    "<Response [201]>"は、POSTリクエストが成功したことを示します。  
    詳細については、[レスポンスステータスコード](https://developer.mozilla.org/ja/docs/Web/HTTP/Status)を参照してください。  

